@@ -798,7 +798,39 @@ IDnum readStarts(Node * node)
 	return sum;
 }
 
-void displayGeneralStatistics(Graph * graph, char *filename)
+static void printShortCounts(FILE * outfile, Node * node, Graph * graph, ReadSet * reads) {
+	IDnum counts[CATEGORIES];
+	Category cat;
+	IDnum shortReadIndex;
+	IDnum readID;
+	IDnum shortReadCount;
+	ShortReadMarker *array;
+	ShortReadMarker *marker;
+
+	if (!readStartsAreActivated(graph)) {
+		for (cat = 0; cat < CATEGORIES; cat++)
+			fprintf(outfile, "\tN/A");
+		return;
+	}
+
+	shortReadCount = getNodeReadCount(node, graph);
+	array = getNodeReads(node, graph);
+
+	for (cat = 0; cat < CATEGORIES; cat++)
+		counts[cat] = 0;
+
+	for (shortReadIndex = 0; shortReadIndex < shortReadCount; shortReadIndex++) {
+		marker = getShortReadMarkerAtIndex(array, shortReadIndex);
+		readID = getShortReadMarkerID(marker);
+		cat = reads->categories[readID - 1] / 2;
+		counts[cat]++;
+	}
+
+	for (cat = 0; cat < CATEGORIES; cat++)
+		fprintf(outfile, "\t%li", (long) counts[cat]);
+}
+
+void displayGeneralStatistics(Graph * graph, char *filename, ReadSet * reads)
 {
 	IDnum nodeIndex;
 	Node *node;
@@ -819,6 +851,11 @@ void displayGeneralStatistics(Graph * graph, char *filename)
 		fprintf(outfile, "\tshort%i_Ocov", (int) (cat + 1));
 	}
 
+	fprintf(outfile, "\tlong_nb");
+	for (cat = 0; cat < CATEGORIES; cat++) {
+		fprintf(outfile, "\tshort%i_nb", (int) (cat + 1));
+	}
+
 	fprintf(outfile, "\n");
 
 	for (nodeIndex = 1; nodeIndex <= nodeCount(graph); nodeIndex++) {
@@ -850,6 +887,9 @@ void displayGeneralStatistics(Graph * graph, char *filename)
 				fprintf(outfile, "\tInf\tInf");
 		}
 
+		fprintf(outfile, "\t%li", (long) markerCount(node));
+		printShortCounts(outfile, node, graph, reads); 
+
 		fprintf(outfile, "\n");
 	}
 
@@ -1493,13 +1533,26 @@ int compareNodeCovs(const void * A, const void * B) {
 	return -1;
 }
 
-double estimated_cov(Graph * graph)
+double estimated_cov(Graph * graph, char * directory)
 {
 	Node ** nodeArray = callocOrExit(nodeCount(graph), Node*); 
 	IDnum index;
 	Coordinate halfTotalLength = 0;
 	Coordinate sumLength = 0;
 	Node *node;
+	char *logFilename =
+	    mallocOrExit(strlen(directory) + 100, char);
+	char *statsLine = 
+	    mallocOrExit(5000, char);
+	FILE *logFile;
+
+	strcpy(logFilename, directory);
+	strcat(logFilename, "/Log");
+	logFile = fopen(logFilename, "a");
+
+	if (logFile == NULL)
+		exitErrorf(EXIT_FAILURE, true, "Could not write to %s",
+		       logFilename);
 
 	puts("Measuring median coverage depth...");
 
@@ -1527,13 +1580,21 @@ double estimated_cov(Graph * graph)
 		sumLength += getNodeLength(node);
 		if (sumLength >= halfTotalLength) {
 			printf("Median coverage depth = %f\n", getTotalCoverage(node) / (double) getNodeLength(node));
+			fprintf(logFile, "Median coverage depth = %f\n", getTotalCoverage(node) / (double) getNodeLength(node));
 			free(nodeArray);
+			fclose(logFile);
+			free(logFilename);
+			free(statsLine);
 			return getTotalCoverage(node) / (double) getNodeLength(node);
 		}
 	}
 
 	// In case something went wrong...
 	free(nodeArray);
+	fclose(logFile);
+	free(logFilename);
+	free(statsLine);
+
 	return -1;
 }
 
